@@ -1,19 +1,33 @@
+/*
+ * Copyright 2011, Ben Langmead <langmea@cs.jhu.edu>
+ *
+ * This file is part of Bowtie 2.
+ *
+ * Bowtie 2 is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Bowtie 2 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Bowtie 2.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #ifndef DIFF_SAMPLE_H_
 #define DIFF_SAMPLE_H_
 
-#if (__cplusplus >= 201103)
-#include <thread>
-#endif
-
 #include <stdint.h>
-
+#include <string.h>
 #include "assert_helpers.h"
-#include "btypes.h"
-#include "ds.h"
-#include "ls.h"
 #include "multikey_qsort.h"
 #include "timer.h"
-#include "threading.h"
+#include "ds.h"
+#include "mem_ids.h"
+#include "ls.h"
 
 using namespace std;
 
@@ -51,8 +65,8 @@ struct sampleEntry {
 
 /// Array of Colbourn and Ling calculated difference covers up to
 /// r = 16 (maxV = 5953)
-static struct sampleEntry clDCs[16];
-static bool clDCs_calced = false; /// have clDCs been calculated?
+extern struct sampleEntry clDCs[16];
+extern bool clDCs_calced; /// have clDCs been calculated?
 
 /**
  * Check that the given difference cover 'ds' actually covers all
@@ -61,7 +75,7 @@ static bool clDCs_calced = false; /// have clDCs been calculated?
 template<typename T>
 static bool dcRepOk(T v, EList<T>& ds) {
 	// diffs[] records all the differences observed
-	bool *covered = new bool[v];
+	AutoArray<bool> covered(v, EBWT_CAT);
 	for(T i = 1; i < v; i++) {
 		covered[i] = false;
 	}
@@ -83,7 +97,6 @@ static bool dcRepOk(T v, EList<T>& ds) {
 			break;
 		}
 	}
-	delete[] covered;
 	return ok;
 }
 
@@ -126,7 +139,7 @@ static inline bool hasDifference(T *ds, T d, T v, T diff) {
 template<typename T>
 void calcExhaustiveDC(T i, bool verbose = false, bool sanityCheck = false) {
 	T v = i;
-	bool *diffs = new bool[v];
+	AutoArray<bool> diffs(v, EBWT_CAT);
 	// v is the target period
 	T ld = (T)ceil(sqrt(v));
 	// ud is the upper bound on |D|
@@ -137,7 +150,7 @@ void calcExhaustiveDC(T i, bool verbose = false, bool sanityCheck = false) {
 	T d;
 	for(d = ld; d <= ud+1; d++) {
 		// for all possible |D| samples
-		ds = new T[d];
+		AutoArray<T> ds(d, EBWT_CAT);
 		for(T j = 0; j < d; j++) {
 			ds[j] = j;
 		}
@@ -158,14 +171,8 @@ void calcExhaustiveDC(T i, bool verbose = false, bool sanityCheck = false) {
 					assert_lt(d2, v);
 					assert_gt(d1, 0);
 					assert_gt(d2, 0);
-					if(!diffs[d1]) {
-						diffCnt++;
-						diffs[d1] = true;
-					}
-					if(!diffs[d2]) {
-						diffCnt++;
-						diffs[d2] = true;
-					}
+					if(!diffs[d1]) diffCnt++; diffs[d1] = true;
+					if(!diffs[d2]) diffCnt++; diffs[d2] = true;
 				}
 			}
 			// Do we observe all possible differences (except 0)
@@ -226,10 +233,8 @@ void calcExhaustiveDC(T i, bool verbose = false, bool sanityCheck = false) {
 		if(ok) {
 			break;
 		}
-		delete[] ds;
 	} // next |D|
 	assert(ok);
-	delete[] diffs;
 	cout << "Did exhaustive v=" << v << " |D|=" << d << endl;
 	cout << "  ";
 	for(T i = 0; i < d; i++) {
@@ -237,7 +242,6 @@ void calcExhaustiveDC(T i, bool verbose = false, bool sanityCheck = false) {
 		if(i < d-1) cout << ",";
 	}
 	cout << endl;
-	delete[] ds;
 }
 
 /**
@@ -282,7 +286,7 @@ void calcColbournAndLingDCs(bool verbose = false, bool sanityCheck = false) {
 		assert_lt(i, 128);
 		if(sanityCheck) {
 			// diffs[] records all the differences observed
-			bool *diffs = new bool[maxv];
+			AutoArray<bool> diffs(maxv, EBWT_CAT);
 			for(T i = 0; i < numsamp; i++) {
 				for(T j = i+1; j < numsamp; j++) {
 					T d1 = (clDCs[r].samples[j] - clDCs[r].samples[i]);
@@ -298,117 +302,28 @@ void calcColbournAndLingDCs(bool verbose = false, bool sanityCheck = false) {
 				if(diffs[i] == false) cout << r << ", " << i << endl;
 				assert(diffs[i] == true);
 			}
-			delete[] diffs;
 		}
 	}
 	clDCs_calced = true;
 }
 
 /**
- * Entries 4-57 are transcribed from page 6 of Luk and Wong's paper
- * "Two New Quorum Based Algorithms for Distributed Mutual Exclusion",
- * which is also used and cited in the Burkhardt and Karkkainen's
- * papers on difference covers for sorting.  These samples are optimal
- * according to Luk and Wong.
- *
- * All other entries are generated via the exhaustive algorithm in
- * calcExhaustiveDC().
- *
- * The 0 is stored at the end of the sample as an end-of-list marker,
- * but 0 is also an element of each.
- *
- * Note that every difference cover has a 0 and a 1.  Intuitively,
- * any optimal difference cover sample can be oriented (i.e. rotated)
- * such that it includes 0 and 1 as elements.
- *
- * All samples in this list have been verified to be complete covers.
- *
- * A value of 0xffffffff in the first column indicates that there is no
- * sample for that value of v.  We do not keep samples for values of v
- * less than 3, since they are trivial (and the caller probably didn't
- * mean to ask for it).
+ * A precalculated list of difference covers.
  */
-static uint32_t dc0to64[65][10] = {
-	{0xffffffff},                     // 0
-	{0xffffffff},                     // 1
-	{0xffffffff},                     // 2
-	{1, 0},                           // 3
-	{1, 2, 0},                        // 4
-	{1, 2, 0},                        // 5
-	{1, 3, 0},                        // 6
-	{1, 3, 0},                        // 7
-	{1, 2, 4, 0},                     // 8
-	{1, 2, 4, 0},                     // 9
-	{1, 2, 5, 0},                     // 10
-	{1, 2, 5, 0},                     // 11
-	{1, 3, 7, 0},                     // 12
-	{1, 3, 9, 0},                     // 13
-	{1, 2, 3, 7, 0},                  // 14
-	{1, 2, 3, 7, 0},                  // 15
-	{1, 2, 5, 8, 0},                  // 16
-	{1, 2, 4, 12, 0},                 // 17
-	{1, 2, 5, 11, 0},                 // 18
-	{1, 2, 6, 9, 0},                  // 19
-	{1, 2, 3, 6, 10, 0},              // 20
-	{1, 4, 14, 16, 0},                // 21
-	{1, 2, 3, 7, 11, 0},              // 22
-	{1, 2, 3, 7, 11, 0},              // 23
-	{1, 2, 3, 7, 15, 0},              // 24
-	{1, 2, 3, 8, 12, 0},              // 25
-	{1, 2, 5, 9, 15, 0},              // 26
-	{1, 2, 5, 13, 22, 0},             // 27
-	{1, 4, 15, 20, 22, 0},            // 28
-	{1, 2, 3, 4, 9, 14, 0},           // 29
-	{1, 2, 3, 4, 9, 19, 0},           // 30
-	{1, 3, 8, 12, 18, 0},             // 31
-	{1, 2, 3, 7, 11, 19, 0},          // 32
-	{1, 2, 3, 6, 16, 27, 0},          // 33
-	{1, 2, 3, 7, 12, 20, 0},          // 34
-	{1, 2, 3, 8, 12, 21, 0},          // 35
-	{1, 2, 5, 12, 14, 20, 0},         // 36
-	{1, 2, 4, 10, 15, 22, 0},         // 37
-	{1, 2, 3, 4, 8, 14, 23, 0},       // 38
-	{1, 2, 4, 13, 18, 33, 0},         // 39
-	{1, 2, 3, 4, 9, 14, 24, 0},       // 40
-	{1, 2, 3, 4, 9, 15, 25, 0},       // 41
-	{1, 2, 3, 4, 9, 15, 25, 0},       // 42
-	{1, 2, 3, 4, 10, 15, 26, 0},      // 43
-	{1, 2, 3, 6, 16, 27, 38, 0},      // 44
-	{1, 2, 3, 5, 12, 18, 26, 0},      // 45
-	{1, 2, 3, 6, 18, 25, 38, 0},      // 46
-	{1, 2, 3, 5, 16, 22, 40, 0},      // 47
-	{1, 2, 5, 9, 20, 26, 36, 0},      // 48
-	{1, 2, 5, 24, 33, 36, 44, 0},     // 49
-	{1, 3, 8, 17, 28, 32, 38, 0},     // 50
-	{1, 2, 5, 11, 18, 30, 38, 0},     // 51
-	{1, 2, 3, 4, 6, 14, 21, 30, 0},   // 52
-	{1, 2, 3, 4, 7, 21, 29, 44, 0},   // 53
-	{1, 2, 3, 4, 9, 15, 21, 31, 0},   // 54
-	{1, 2, 3, 4, 6, 19, 26, 47, 0},   // 55
-	{1, 2, 3, 4, 11, 16, 33, 39, 0},  // 56
-	{1, 3, 13, 32, 36, 43, 52, 0},    // 57
-
-	// Generated by calcExhaustiveDC()
-	{1, 2, 3, 7, 21, 33, 37, 50, 0},  // 58
-	{1, 2, 3, 6, 13, 21, 35, 44, 0},  // 59
-	{1, 2, 4, 9, 15, 25, 30, 42, 0},  // 60
-	{1, 2, 3, 7, 15, 25, 36, 45, 0},  // 61
-	{1, 2, 4, 10, 32, 39, 46, 51, 0}, // 62
-	{1, 2, 6, 8, 20, 38, 41, 54, 0},  // 63
-	{1, 2, 5, 14, 16, 34, 42, 59, 0}  // 64
-};
+extern uint32_t dc0to64[65][10];
 
 /**
  * Get a difference cover for the requested periodicity v.
  */
 template <typename T>
-static EList<T> getDiffCover(T v,
-                              bool verbose = false,
-                              bool sanityCheck = false)
+static EList<T> getDiffCover(
+	T v,
+	bool verbose = false,
+	bool sanityCheck = false)
 {
 	assert_gt(v, 2);
 	EList<T> ret;
-
+	ret.clear();
 	// Can we look it up in our hardcoded array?
 	if(v <= 64 && dc0to64[v][0] == 0xffffffff) {
 		if(verbose) cout << "v in hardcoded area, but hardcoded entry was all-fs" << endl;
@@ -537,13 +452,13 @@ public:
 		_isaPrime(),
 		_dInv(),
 		_log2v(myLog2(_v)),
-		_vmask(OFF_MASK << _log2v),
+		_vmask(0xffffffff << _log2v),
 		_logger(__logger)
 	{
 		assert_gt(_d, 0);
 		assert_eq(1, popCount(_v)); // must be power of 2
 		// Build map from d's to idx's
-		_dInv.resizeExact((size_t)_v);
+		_dInv.resizeExact((size_t)v());
 		_dInv.fill(0xffffffff);
 		uint32_t lim = (uint32_t)_ds.size();
 		for(uint32_t i = 0; i < lim; i++) {
@@ -558,33 +473,33 @@ public:
 	 * the approximate number of bytes the Cover takes at all times.
 	 */
 	static size_t simulateAllocs(const TStr& text, uint32_t v) {
-		EList<uint32_t> ds = getDiffCover(v, false /*verbose*/, false /*sanity*/);
+		EList<uint32_t> ds(getDiffCover(v, false /*verbose*/, false /*sanity*/));
 		size_t len = text.length();
 		size_t sPrimeSz = (len / v) * ds.size();
 		// sPrime, sPrimeOrder, _isaPrime all exist in memory at
 		// once and that's the peak
-		AutoArray<TIndexOffU> aa(sPrimeSz * 3 + (1024 * 1024 /*out of caution*/));
+		AutoArray<uint32_t> aa(sPrimeSz * 3 + (1024 * 1024 /*out of caution*/), EBWT_CAT);
 		return sPrimeSz * 4; // sPrime array
 	}
 
 	uint32_t v() const                   { return _v; }
 	uint32_t log2v() const               { return _log2v; }
 	uint32_t vmask() const               { return _vmask; }
-	uint32_t modv(TIndexOffU i) const      { return (uint32_t)(i & ~_vmask); }
-	TIndexOffU divv(TIndexOffU i) const      { return i >> _log2v; }
+	uint32_t modv(uint32_t i) const      { return i & ~_vmask; }
+	uint32_t divv(uint32_t i) const      { return i >> _log2v; }
 	uint32_t d() const                   { return _d; }
 	bool verbose() const                 { return _verbose; }
 	bool sanityCheck() const             { return _sanity; }
 	const TStr& text() const             { return _text; }
-	const EList<uint32_t>& ds() const   { return _ds; }
-	const EList<uint32_t>& dmap() const { return _dmap; }
+	const EList<uint32_t>& ds() const    { return _ds; }
+	const EList<uint32_t>& dmap() const  { return _dmap; }
 	ostream& log() const                 { return _logger; }
 
-	void     build(int nthreads);
-	uint32_t tieBreakOff(TIndexOffU i, TIndexOffU j) const;
-	int64_t  breakTie(TIndexOffU i, TIndexOffU j) const;
-	bool     isCovered(TIndexOffU i) const;
-	TIndexOffU rank(TIndexOffU i) const;
+	void     build();
+	uint32_t tieBreakOff(uint32_t i, uint32_t j) const;
+	int64_t  breakTie(uint32_t i, uint32_t j) const;
+	bool     isCovered(uint32_t i) const;
+	uint32_t rank(uint32_t i) const;
 
 	/**
 	 * Print out the suffix array such that every sample offset has its
@@ -607,7 +522,7 @@ public:
 private:
 
 	void doBuiltSanityCheck() const;
-	void buildSPrime(EList<TIndexOffU>& sPrime, size_t padding);
+	void buildSPrime(EList<uint32_t>& sPrime, size_t padding);
 
 	bool built() const {
 		return _isaPrime.size() > 0;
@@ -624,39 +539,17 @@ private:
 	uint32_t         _v;        // periodicity of sample
 	bool             _verbose;  //
 	bool             _sanity;   //
-	EList<uint32_t> _ds;       // samples: idx -> d
-	EList<uint32_t> _dmap;     // delta map
+	EList<uint32_t>  _ds;       // samples: idx -> d
+	EList<uint32_t>  _dmap;     // delta map
 	uint32_t         _d;        // |D| - size of sample
-	EList<TIndexOffU> _doffs;    // offsets into sPrime/isaPrime for each d idx
-	EList<TIndexOffU> _isaPrime; // ISA' array
-	EList<uint32_t> _dInv;     // Map from d -> idx
+	EList<uint32_t>  _doffs;    // offsets into sPrime/isaPrime for each d idx
+	EList<uint32_t>  _isaPrime; // ISA' array
+	EList<uint32_t>  _dInv;     // Map from d -> idx
 	uint32_t         _log2v;
-	TIndexOffU         _vmask;
+	uint32_t         _vmask;
 	ostream&         _logger;
 };
 
-/**
- * Return true iff suffixes with offsets suf1 and suf2 out of host
- * string 'host' are identical up to depth 'v'.
- */
-template <typename TStr>
-static inline bool suffixLt(const TStr& host, TIndexOffU suf1, TIndexOffU suf2) {
-	TIndexOffU hlen = (TIndexOffU)host.length();
-	assert_neq(suf1, suf2);
-	TIndexOffU i = 0;
-	while(suf1 + i < hlen && suf2 + i < hlen) {
-		if(host[suf1+i] < host[suf2+i]) return true;
-		if(host[suf1+i] > host[suf2+i]) return false;
-		i++;
-	}
-	if(suf1 + i == hlen) {
-		assert_lt(suf2 + i, hlen);
-		return false;
-	}
-	assert_eq(suf2 + i, hlen);
-	return true;
-}
-
 /**
  * Sanity-check the difference cover by first inverting _isaPrime then
  * checking that each successive suffix really is less than the next.
@@ -666,24 +559,26 @@ void DifferenceCoverSample<TStr>::doBuiltSanityCheck() const {
 	uint32_t v = this->v();
 	assert(built());
 	VMSG_NL("  Doing sanity check");
-	TIndexOffU added = 0;
-	EList<TIndexOffU> sorted;
+	uint32_t added = 0;
+	EList<uint32_t> sorted;
 	sorted.resizeExact(_isaPrime.size());
-	sorted.fill(OFF_MASK);
+	sorted.fill(0xffffffff);
 	for(size_t di = 0; di < this->d(); di++) {
 		uint32_t d = _ds[di];
 		size_t i = 0;
 		for(size_t doi = _doffs[di]; doi < _doffs[di+1]; doi++, i++) {
-			assert_eq(OFF_MASK, sorted[_isaPrime[doi]]);
+			assert_eq(0xffffffff, sorted[_isaPrime[doi]]);
 			// Maps the offset of the suffix to its rank
-			sorted[_isaPrime[doi]] = (TIndexOffU)(v*i + d);
+			sorted[_isaPrime[doi]] = (uint32_t)(v*i + d);
 			added++;
 		}
 	}
 	assert_eq(added, _isaPrime.size());
+#ifndef NDEBUG
 	for(size_t i = 0; i < sorted.size()-1; i++) {
-		assert(suffixLt(this->text(), sorted[i], sorted[i+1]));
+		assert(sstr_suf_lt(this->text(), sorted[i], this->text(), sorted[i+1], false));
 	}
+#endif
 }
 
 /**
@@ -696,25 +591,25 @@ void DifferenceCoverSample<TStr>::doBuiltSanityCheck() const {
  */
 template <typename TStr>
 void DifferenceCoverSample<TStr>::buildSPrime(
-	EList<TIndexOffU>& sPrime,
+	EList<uint32_t>& sPrime,
 	size_t padding)
 {
 	const TStr& t = this->text();
 	const EList<uint32_t>& ds = this->ds();
-	TIndexOffU tlen = (TIndexOffU)t.length();
+	uint32_t tlen = (uint32_t)t.length();
 	uint32_t v = this->v();
 	uint32_t d = this->d();
 	assert_gt(v, 2);
 	assert_lt(d, v);
 	// Record where each d section should begin in sPrime
-	TIndexOffU tlenDivV = this->divv(tlen);
+	uint32_t tlenDivV = this->divv(tlen);
 	uint32_t tlenModV = this->modv(tlen);
-	TIndexOffU sPrimeSz = 0;
+	uint32_t sPrimeSz = 0;
 	assert(_doffs.empty());
-	_doffs.resize(d+1);
+	_doffs.resizeExact((size_t)d+1);
 	for(uint32_t di = 0; di < d; di++) {
 		// mu mapping
-		TIndexOffU sz = tlenDivV + ((ds[di] <= tlenModV) ? 1 : 0);
+		uint32_t sz = tlenDivV + ((ds[di] <= tlenModV) ? 1 : 0);
 		assert_geq(sz, 0);
 		_doffs[di] = sPrimeSz;
 		sPrimeSz += sz;
@@ -724,7 +619,7 @@ void DifferenceCoverSample<TStr>::buildSPrime(
 	if(tlenDivV > 0) {
 		for(size_t i = 0; i < d; i++) {
 			assert_gt(_doffs[i+1], _doffs[i]);
-			TIndexOffU diff = _doffs[i+1] - _doffs[i];
+			uint32_t diff = _doffs[i+1] - _doffs[i];
 			assert(diff == tlenDivV || diff == tlenDivV+1);
 		}
 	}
@@ -732,20 +627,20 @@ void DifferenceCoverSample<TStr>::buildSPrime(
 	assert_eq(_doffs.size(), d+1);
 	// Size sPrime appropriately
 	sPrime.resizeExact((size_t)sPrimeSz + padding);
-	sPrime.fill(OFF_MASK);
+	sPrime.fill(0xffffffff);
 	// Slot suffixes from text into sPrime according to the mu
 	// mapping; where the mapping would leave a blank, insert a 0
-	TIndexOffU added = 0;
-	TIndexOffU i = 0;
-	for(uint64_t ti = 0; ti <= tlen; ti += v) {
+	uint32_t added = 0;
+	uint32_t i = 0;
+	for(uint32_t ti = 0; ti <= tlen; ti += v) {
 		for(uint32_t di = 0; di < d; di++) {
-			TIndexOffU tti = (TIndexOffU)(ti + ds[di]);
+			uint32_t tti = ti + ds[di];
 			if(tti > tlen) break;
-			TIndexOffU spi = _doffs[di] + i;
+			uint32_t spi = _doffs[di] + i;
 			assert_lt(spi, _doffs[di+1]);
 			assert_leq(tti, tlen);
 			assert_lt(spi, sPrimeSz);
-			assert_eq(OFF_MASK, sPrime[spi]);
+			assert_eq(0xffffffff, sPrime[spi]);
 			sPrime[spi] = tti; added++;
 		}
 		i++;
@@ -760,11 +655,11 @@ void DifferenceCoverSample<TStr>::buildSPrime(
 template <typename TStr>
 static inline bool suffixSameUpTo(
 	const TStr& host,
-	TIndexOffU suf1,
-	TIndexOffU suf2,
-	TIndexOffU v)
+	uint32_t suf1,
+	uint32_t suf2,
+	uint32_t v)
 {
-	for(TIndexOffU i = 0; i < v; i++) {
+	for(uint32_t i = 0; i < v; i++) {
 		bool endSuf1 = suf1+i >= host.length();
 		bool endSuf2 = suf2+i >= host.length();
 		if((endSuf1 && !endSuf2) || (!endSuf1 && endSuf2)) return false;
@@ -774,58 +669,12 @@ static inline bool suffixSameUpTo(
 	return true;
 }
 
-template<typename TStr>
-struct VSortingParam {
-        DifferenceCoverSample<TStr>* dcs;
-        TIndexOffU*                  sPrimeArr;
-        size_t                       sPrimeSz;
-        TIndexOffU*                  sPrimeOrderArr;
-        size_t                       depth;
-        const EList<size_t>*   boundaries;
-        size_t*                      cur;
-        MUTEX_T*                     mutex;
-};
-
-template<typename TStr>
-static void VSorting_worker(void *vp)
-{
-        VSortingParam<TStr>* param = (VSortingParam<TStr>*)vp;
-	DifferenceCoverSample<TStr>* dcs = param->dcs;
-	const TStr& host = dcs->text();
-	const size_t hlen = host.length();
-	uint32_t v = dcs->v();
-	while(true) {
-	  size_t cur = 0;
-	  {
-	    ThreadSafe ts(param->mutex);
-	    cur = *(param->cur);
-	    (*param->cur)++;
-	  }
-	  if(cur >= param->boundaries->size()) return;
-	  size_t begin = (cur == 0 ? 0 : (*param->boundaries)[cur-1]);
-	  size_t end = (*param->boundaries)[cur];
-	  assert_leq(begin, end);
-	  if(end - begin <= 1) continue;
-	  mkeyQSortSuf2(
-			host,
-			hlen,
-			param->sPrimeArr,
-			param->sPrimeSz,
-			param->sPrimeOrderArr,
-			4,
-			begin,
-			end,
-			param->depth,
-			v);
-	}
-}
-
 /**
  * Calculates a ranking of all suffixes in the sample and stores them,
  * packed according to the mu mapping, in _isaPrime.
  */
 template <typename TStr>
-void DifferenceCoverSample<TStr>::build(int nthreads) {
+void DifferenceCoverSample<TStr>::build() {
 	// Local names for relevant types
 	VMSG_NL("Building DifferenceCoverSample");
 	// Local names for relevant data
@@ -833,19 +682,22 @@ void DifferenceCoverSample<TStr>::build(int nthreads) {
 	uint32_t v = this->v();
 	assert_gt(v, 2);
 	// Build s'
+	EList<uint32_t> sPrime;
+	// Need to allocate 2 extra elements at the end of the sPrime and _isaPrime
+	// arrays.  One element that's less than all others, and another that acts
+	// as needed padding for the Larsson-Sadakane sorting code.
 	size_t padding = 1;
-	EList<TIndexOffU> sPrime;
 	VMSG_NL("  Building sPrime");
 	buildSPrime(sPrime, padding);
 	size_t sPrimeSz = sPrime.size() - padding;
-	assert_gt(sPrime.size(), 0);
-	assert_leq(sPrime.size(), t.length()+1); // +1 is because of the end-cap
-	TIndexOffU nextRank = 0;
+	assert_gt(sPrime.size(), padding);
+	assert_leq(sPrime.size(), t.length() + padding + 1);
+	uint32_t nextRank = 0;
 	{
 		VMSG_NL("  Building sPrimeOrder");
-		EList<TIndexOffU> sPrimeOrder;
+		EList<uint32_t> sPrimeOrder;
 		sPrimeOrder.resizeExact(sPrimeSz);
-		for(TIndexOffU i = 0; i < (TIndexOffU)sPrimeOrder.size(); i++) {
+		for(uint32_t i = 0; i < sPrimeSz; i++) {
 			sPrimeOrder[i] = i;
 		}
 		// sPrime now holds suffix-offsets for DC samples.
@@ -855,18 +707,12 @@ void DifferenceCoverSample<TStr>::build(int nthreads) {
 			// Extract backing-store array from sPrime and sPrimeOrder;
 			// the mkeyQSortSuf2 routine works on the array for maximum
 			// efficiency
-			TIndexOffU *sPrimeArr = (TIndexOffU*)sPrime.ptr();
+			uint32_t *sPrimeArr = (uint32_t*)sPrime.ptr();
 			assert_eq(sPrimeArr[0], sPrime[0]);
 			assert_eq(sPrimeArr[sPrimeSz-1], sPrime[sPrimeSz-1]);
-			TIndexOffU *sPrimeOrderArr = (TIndexOffU*)sPrimeOrder.ptr();
+			uint32_t *sPrimeOrderArr = (uint32_t*)sPrimeOrder.ptr();
 			assert_eq(sPrimeOrderArr[0], sPrimeOrder[0]);
 			assert_eq(sPrimeOrderArr[sPrimeSz-1], sPrimeOrder[sPrimeSz-1]);
-
-			TIndexOffU *sOrig = NULL;
-			if(this->sanityCheck()) {
-				sOrig = new TIndexOffU[sPrimeSz];
-				memcpy(sOrig, sPrimeArr, OFF_SIZE * sPrimeSz);
-			}
 			// Sort sample suffixes up to the vth character using a
 			// multikey quicksort.  Sort time is proportional to the
 			// number of samples times v.  It isn't quadratic.
@@ -875,58 +721,8 @@ void DifferenceCoverSample<TStr>::build(int nthreads) {
 			// elements in sPrime, it swaps the same elements in
 			// sPrimeOrder too.  This allows us to easily reconstruct
 			// what the sort did.
-			if(nthreads == 1) {
-				mkeyQSortSuf2(t, sPrimeArr, sPrimeSz, sPrimeOrderArr, 4,
-				              this->verbose(), this->sanityCheck(), v);
-			} else {
-				int query_depth = 0;
-				int tmp_nthreads = nthreads;
-				while(tmp_nthreads > 0) {
-					query_depth++;
-					tmp_nthreads >>= 1;
-				}
-				EList<size_t> boundaries; // bucket boundaries for parallelization
-				mkeyQSortSuf2(t, sPrimeArr, sPrimeSz, sPrimeOrderArr, 4,
-				              this->verbose(), false, query_depth, &boundaries);
-				if(boundaries.size() > 0) {
-#if (__cplusplus >= 201103)
-					AutoArray<std::thread*> threads(nthreads);
-#else
-					AutoArray<tthread::thread*> threads(nthreads);
-#endif
-					EList<VSortingParam<TStr> > tparams;
-					tparams.resize(nthreads);
-					size_t cur = 0;
-					MUTEX_T mutex;
-					for(int tid = 0; tid < nthreads; tid++) {
-						// Calculate bucket sizes by doing a binary search for each
-						// suffix and noting where it lands
-						tparams[tid].dcs = this;
-						tparams[tid].sPrimeArr = sPrimeArr;
-						tparams[tid].sPrimeSz = sPrimeSz;
-						tparams[tid].sPrimeOrderArr = sPrimeOrderArr;
-						tparams[tid].depth = query_depth;
-						tparams[tid].boundaries = &boundaries;
-						tparams[tid].cur = &cur;
-						tparams[tid].mutex = &mutex;
-#if (__cplusplus >= 201103L)
-						threads[tid] = new std::thread(VSorting_worker<TStr>, (void*)&tparams[tid]);
-#else
-						threads[tid] = new tthread::thread(VSorting_worker<TStr>, (void*)&tparams[tid]);
-#endif
-					}
-					for (int tid = 0; tid < nthreads; tid++) {
-						threads[tid]->join();
-					}
-				}
-			}
-			if(this->sanityCheck()) {
-				sanityCheckOrderedSufs(t, t.length(), sPrimeArr, sPrimeSz, v);
-				for(size_t i = 0; i < sPrimeSz; i++) {
-					assert_eq(sPrimeArr[i], sOrig[sPrimeOrderArr[i]]);
-				}
-				delete[] sOrig;
-			}
+			mkeyQSortSuf2(t, sPrimeArr, sPrimeSz, sPrimeOrderArr, 4,
+			              this->verbose(), this->sanityCheck(), v);
 			// Make sure sPrime and sPrimeOrder are consistent with
 			// their respective backing-store arrays
 			assert_eq(sPrimeArr[0], sPrime[0]);
@@ -937,9 +733,8 @@ void DifferenceCoverSample<TStr>::build(int nthreads) {
 		// Now assign the ranking implied by the sorted sPrime/sPrimeOrder
 		// arrays back into sPrime.
 		VMSG_NL("  Allocating rank array");
-		_isaPrime.resize(sPrime.size());
 		_isaPrime.resizeExact(sPrime.size());
-		ASSERT_ONLY(_isaPrime.fill(OFF_MASK));
+		ASSERT_ONLY(_isaPrime.fill(0xffffffff));
 		assert_gt(_isaPrime.size(), 0);
 		{
 			Timer timer(cout, "  Ranking v-sort output time: ", this->verbose());
@@ -952,45 +747,47 @@ void DifferenceCoverSample<TStr>::build(int nthreads) {
 				if(!suffixSameUpTo(t, sPrime[i], sPrime[i+1], v)) nextRank++;
 			}
 			_isaPrime[sPrimeOrder[sPrimeSz-1]] = nextRank; // finish off
+#ifndef NDEBUG
+			for(size_t i = 0; i < sPrimeSz; i++) {
+				assert_neq(0xffffffff, _isaPrime[i]);
+				assert_lt(_isaPrime[i], sPrimeSz);
+			}
+#endif
 		}
 		// sPrimeOrder is destroyed
 		// All the information we need is now in _isaPrime
 	}
-#ifndef NDEBUG
-	// Check that all ranks are sane
-	for(size_t i = 0; i < sPrimeSz; i++) {
-		assert_neq(_isaPrime[i], OFF_MASK);
-		assert_lt(_isaPrime[i], sPrimeSz);
-	}
-#endif
-	// Now pass the sPrimeRanks[] array to LarssonSadakane (in SeqAn).
-	_isaPrime[_isaPrime.size() - 1] = sPrimeSz;
-	sPrime[sPrime.size() - 1] = sPrimeSz;
+	_isaPrime[_isaPrime.size()-1] = (uint32_t)sPrimeSz;
+	sPrime[sPrime.size()-1] = (uint32_t)sPrimeSz;
+	// _isaPrime[_isaPrime.size()-1] and sPrime[sPrime.size()-1] are just
+	// spacer for the Larsson-Sadakane routine to use
 	{
 		Timer timer(cout, "  Invoking Larsson-Sadakane on ranks time: ", this->verbose());
 		VMSG_NL("  Invoking Larsson-Sadakane on ranks");
-		LarssonSadakane<TIndexOff> ls;
+		if(sPrime.size() >= 0x10000000) {
+			cerr << "Error; sPrime array has so many elements that it can't be converted to a signed array without overflow." << endl;
+			throw 1;
+		}
+		LarssonSadakane<int> ls;
 		ls.suffixsort(
-			(TIndexOff*)&_isaPrime[0],
-			(TIndexOff*)&sPrime[0],
-			(TIndexOff)sPrimeSz,
-			(TIndexOff)sPrime.size(),
+			(int*)_isaPrime.ptr(),
+			(int*)sPrime.ptr(),
+			(int)sPrimeSz,
+			(int)sPrime.size(),
 			0);
 	}
-	// sPrime now contains the suffix array (which we ignore)
-	assert_eq(_isaPrime.size(), sPrime.size());
-	assert_gt(_isaPrime.size(), 0);
 	// chop off final character of _isaPrime
-	_isaPrime.resize(sPrimeSz);
-	// Subtract 1 from each isaPrime (to adjust for LarssonSadakane
-	// always ranking the final suffix as lexicographically first)
+	_isaPrime.resizeExact(sPrimeSz);
 	for(size_t i = 0; i < _isaPrime.size(); i++) {
 		_isaPrime[i]--;
 	}
+#ifndef NDEBUG
+	for(size_t i = 0; i < sPrimeSz-1; i++) {
+		assert_lt(_isaPrime[i], sPrimeSz);
+		assert(i == 0 || _isaPrime[i] != _isaPrime[i-1]);
+	}
+#endif
 	VMSG_NL("  Sanity-checking and returning");
-
-	// done!
-	//if(this->verbose()) print(cout);
 	if(this->sanityCheck()) doBuiltSanityCheck();
 }
 
@@ -1000,7 +797,7 @@ void DifferenceCoverSample<TStr>::build(int nthreads) {
  * logic elsewhere.
  */
 template <typename TStr>
-bool DifferenceCoverSample<TStr>::isCovered(TIndexOffU i) const {
+bool DifferenceCoverSample<TStr>::isCovered(uint32_t i) const {
 	assert(built());
 	uint32_t modi = this->modv(i);
 	assert_lt(modi, _dInv.size());
@@ -1012,16 +809,16 @@ bool DifferenceCoverSample<TStr>::isCovered(TIndexOffU i) const {
  * among the sample suffixes.
  */
 template <typename TStr>
-TIndexOffU DifferenceCoverSample<TStr>::rank(TIndexOffU i) const {
+uint32_t DifferenceCoverSample<TStr>::rank(uint32_t i) const {
 	assert(built());
 	assert_lt(i, this->text().length());
 	uint32_t imodv = this->modv(i);
 	assert_neq(0xffffffff, _dInv[imodv]); // must be in the sample
-	TIndexOffU ioff = this->divv(i);
+	uint32_t ioff = this->divv(i);
 	assert_lt(ioff, _doffs[_dInv[imodv]+1] - _doffs[_dInv[imodv]]);
-	TIndexOffU isaIIdx = _doffs[_dInv[imodv]] + ioff;
+	uint32_t isaIIdx = _doffs[_dInv[imodv]] + ioff;
 	assert_lt(isaIIdx, _isaPrime.size());
-	TIndexOffU isaPrimeI = _isaPrime[isaIIdx];
+	uint32_t isaPrimeI = _isaPrime[isaIIdx];
 	assert_leq(isaPrimeI, _isaPrime.size());
 	return isaPrimeI;
 }
@@ -1031,7 +828,7 @@ TIndexOffU DifferenceCoverSample<TStr>::rank(TIndexOffU i) const {
  * if suffix j is lexicographically greater.
  */
 template <typename TStr>
-int64_t DifferenceCoverSample<TStr>::breakTie(TIndexOffU i, TIndexOffU j) const {
+int64_t DifferenceCoverSample<TStr>::breakTie(uint32_t i, uint32_t j) const {
 	assert(built());
 	assert_neq(i, j);
 	assert_lt(i, this->text().length());
@@ -1042,20 +839,20 @@ int64_t DifferenceCoverSample<TStr>::breakTie(TIndexOffU i, TIndexOffU j) const
 	assert_neq(0xffffffff, _dInv[jmodv]); // must be in the sample
 	uint32_t dimodv = _dInv[imodv];
 	uint32_t djmodv = _dInv[jmodv];
-	TIndexOffU ioff = this->divv(i);
-	TIndexOffU joff = this->divv(j);
+	uint32_t ioff = this->divv(i);
+	uint32_t joff = this->divv(j);
 	assert_lt(dimodv+1, _doffs.size());
 	assert_lt(djmodv+1, _doffs.size());
 	// assert_lt: expected (32024) < (0)
 	assert_lt(ioff, _doffs[dimodv+1] - _doffs[dimodv]);
 	assert_lt(joff, _doffs[djmodv+1] - _doffs[djmodv]);
-	TIndexOffU isaIIdx = _doffs[dimodv] + ioff;
-	TIndexOffU isaJIdx = _doffs[djmodv] + joff;
+	uint32_t isaIIdx = _doffs[dimodv] + ioff;
+	uint32_t isaJIdx = _doffs[djmodv] + joff;
 	assert_lt(isaIIdx, _isaPrime.size());
 	assert_lt(isaJIdx, _isaPrime.size());
 	assert_neq(isaIIdx, isaJIdx); // ranks must be unique
-	TIndexOffU isaPrimeI = _isaPrime[isaIIdx];
-	TIndexOffU isaPrimeJ = _isaPrime[isaJIdx];
+	uint32_t isaPrimeI = _isaPrime[isaIIdx];
+	uint32_t isaPrimeJ = _isaPrime[isaJIdx];
 	assert_neq(isaPrimeI, isaPrimeJ); // ranks must be unique
 	assert_leq(isaPrimeI, _isaPrime.size());
 	assert_leq(isaPrimeJ, _isaPrime.size());
@@ -1067,7 +864,7 @@ int64_t DifferenceCoverSample<TStr>::breakTie(TIndexOffU i, TIndexOffU j) const
  * be compared before the difference cover can break the tie.
  */
 template <typename TStr>
-uint32_t DifferenceCoverSample<TStr>::tieBreakOff(TIndexOffU i, TIndexOffU j) const {
+uint32_t DifferenceCoverSample<TStr>::tieBreakOff(uint32_t i, uint32_t j) const {
 	const TStr& t = this->text();
 	const EList<uint32_t>& dmap = this->dmap();
 	assert(built());
