@@ -40,9 +40,8 @@ public:
 	MemoryTally() : tot_(0), peak_(0) {
 		memset(tots_,  0, 256 * sizeof(uint64_t));
 		memset(peaks_, 0, 256 * sizeof(uint64_t));
-		MUTEX_INIT(lock_);
 	}
-	
+
 	/**
 	 * Tally a memory allocation of size amt bytes.
 	 */
@@ -92,7 +91,7 @@ public:
 
 protected:
 
-	MUTEX_T lock_;
+	MUTEX_T mutex_m;
 	uint64_t tots_[256];
 	uint64_t tot_;
 	uint64_t peaks_[256];
@@ -2948,6 +2947,13 @@ public:
 		return l_.empty();
 	}
 	
+	/**
+	 * Return element at offset i.
+	 */
+	const T& operator[](size_t i) const {
+		return l_[i];
+	}
+	
 #ifndef NDEBUG
 	/**
 	 * Check that heap property holds.
