@@ -32,7 +32,6 @@
 #include "assert_helpers.h"
 #include "tokenize.h"
 #include "random_source.h"
-#include "spinlock.h"
 #include "threading.h"
 #include "filebuf.h"
 #include "qual.h"
@@ -149,9 +148,8 @@ public:
 		numWrappers_(0),
 		doLocking_(true),
 		useSpinlock_(p.useSpinlock),
-		lock_()
+		mutex()
 	{
-		MUTEX_INIT(lock_);
 	}
 
 	virtual ~PatternSource() { }
@@ -231,17 +229,7 @@ public:
 	 */
 	void lock() {
 		if(!doLocking_) return; // no contention
-#ifdef USE_SPINLOCK
-		if(useSpinlock_) {
-			// User can ask to use the normal pthreads lock even if
-			// spinlocks are compiled in.
-			spinlock_.Enter();
-		} else {
-#endif
-			MUTEX_LOCK(lock_);
-#ifdef USE_SPINLOCK
-		}
-#endif
+        mutex.lock();
 	}
 
 	/**
@@ -250,17 +238,7 @@ public:
 	 */
 	void unlock() {
 		if(!doLocking_) return; // no contention
-#ifdef USE_SPINLOCK
-		if(useSpinlock_) {
-			// User can ask to use the normal pthreads lock even if
-			// spinlocks are compiled in.
-			spinlock_.Leave();
-		} else {
-#endif
-			MUTEX_UNLOCK(lock_);
-#ifdef USE_SPINLOCK
-		}
-#endif
+        mutex.unlock();
 	}
 
 	/**
@@ -290,10 +268,7 @@ protected:
 	/// spinlocks is enabled and compiled in.  This is sometimes better
 	/// if we expect bad I/O latency on some reads.
 	bool useSpinlock_;
-#ifdef USE_SPINLOCK
-	SpinLock spinlock_;
-#endif
-	MUTEX_T lock_; /// mutex for locking critical regions
+	MUTEX_T mutex;
 };
 
 /**
@@ -302,9 +277,7 @@ protected:
  */
 class PairedPatternSource {
 public:
-	PairedPatternSource(const PatternParams& p) : seed_(p.seed) {
-		MUTEX_INIT(lock_);
-	}
+	PairedPatternSource(const PatternParams& p) : mutex_m(), seed_(p.seed) {}
 	virtual ~PairedPatternSource() { }
 
 	virtual void addWrapper() = 0;
@@ -327,22 +300,14 @@ public:
 	 * fields is being updated.
 	 */
 	void lock() {
-#ifdef USE_SPINLOCK
-		spinlock_.Enter();
-#else
-		MUTEX_LOCK(lock_);
-#endif
+		mutex_m.lock();
 	}
 
 	/**
 	 * Unlock this PairedPatternSource.
 	 */
 	void unlock() {
-#ifdef USE_SPINLOCK
-		spinlock_.Leave();
-#else
-		MUTEX_UNLOCK(lock_);
-#endif
+		mutex_m.unlock();
 	}
 
 	/**
@@ -363,10 +328,7 @@ public:
 
 protected:
 
-#ifdef USE_SPINLOCK
-	SpinLock spinlock_;
-#endif
-	MUTEX_T lock_; /// mutex for locking critical regions
+	MUTEX_T mutex_m; /// mutex for syncing over critical regions
 	uint32_t seed_;
 };
 
