@@ -1,3 +1,22 @@
+/*
+ * Copyright 2011, Ben Langmead <langmea@cs.jhu.edu>
+ *
+ * This file is part of Bowtie 2.
+ *
+ * Bowtie 2 is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Bowtie 2 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Bowtie 2.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "ref_read.h"
 
 /**
@@ -6,10 +25,11 @@
  * anything with the sequence characters themselves; this is purely for
  * measuring lengths.
  */
-RefRecord fastaRefReadSize(FileBuf& in,
-                           const RefReadInParams& rparms,
-                           bool first,
-                           BitpairOutFileBuf* bpout)
+RefRecord fastaRefReadSize(
+	FileBuf& in,
+	const RefReadInParams& rparms,
+	bool first,
+	BitpairOutFileBuf* bpout)
 {
 	int c;
 	static int lastc = '>'; // last character seen
@@ -58,7 +78,7 @@ RefRecord fastaRefReadSize(FileBuf& in,
 			// Don't emit a warning, since this might legitimately be
 			// a gap on the end of the final sequence in the file
 			lastc = -1;
-			return RefRecord(off, len, first);
+			return RefRecord((uint32_t)off, (uint32_t)len, first);
 		}
 	}
 
@@ -66,8 +86,8 @@ RefRecord fastaRefReadSize(FileBuf& in,
 	// as we go
 	int lc = -1; // last-DNA char variable for color conversion
 	while(true) {
-		int cat = dna4Cat[c];
-		if(rparms.nsToAs && cat == 2) c = 'A';
+		int cat = asc2dnacat[c];
+		if(rparms.nsToAs && cat >= 2) c = 'A';
 		if(cat == 1) {
 			// This is a DNA character
 			if(rparms.color) {
@@ -76,7 +96,7 @@ RefRecord fastaRefReadSize(FileBuf& in,
 					break; // to read-in loop
 				}
 				// Keep going; we need two consecutive unambiguous DNAs
-				lc = charToDna5[(int)c];
+				lc = asc2dna[(int)c];
 				// The 'if(off > 0)' takes care of the case where
 				// the reference is entirely unambiguous and we don't
 				// want to incorrectly increment off.
@@ -84,7 +104,7 @@ RefRecord fastaRefReadSize(FileBuf& in,
 			} else {
 				break; // to read-in loop
 			}
-		} else if(cat == 2) {
+		} else if(cat >= 2) {
 			if(lc != -1 && off == 0) off++;
 			lc = -1;
 			off++; // skip over gap character and increment
@@ -95,7 +115,8 @@ RefRecord fastaRefReadSize(FileBuf& in,
 				cerr << "Warning: Encountered empty reference sequence" << endl;
 			}
 			lastc = '>';
-			return RefRecord(off, 0, first);
+			//return RefRecord(off, 0, false);
+			return RefRecord((uint32_t)off, 0, first);
 		}
 		c = in.get();
 		if(c == -1) {
@@ -106,11 +127,12 @@ RefRecord fastaRefReadSize(FileBuf& in,
 				cerr << "Warning: Encountered empty reference sequence" << endl;
 			}
 			lastc = -1;
-			return RefRecord(off, 0, first);
+			//return RefRecord(off, 0, false);
+			return RefRecord((uint32_t)off, 0, first);
 		}
 	}
 	assert(!rparms.color || (lc != -1));
-	assert_eq(1, dna4Cat[c]); // C must be unambiguous base
+	assert_eq(1, asc2dnacat[c]); // C must be unambiguous base
 	if(off > 0 && rparms.color && first) {
 		// Handle the case where the first record has ambiguous
 		// characters but we're in color space; one of those counts is
@@ -121,8 +143,8 @@ RefRecord fastaRefReadSize(FileBuf& in,
 	// in now points just past the first character of a sequence
 	// line, and c holds the first character
 	while(c != -1 && c != '>') {
-		if(rparms.nsToAs && dna4Cat[c] == 2) c = 'A';
-		uint8_t cat = dna4Cat[c];
+		if(rparms.nsToAs && asc2dnacat[c] >= 2) c = 'A';
+		uint8_t cat = asc2dnacat[c];
 		int cc = toupper(c);
 		if(rparms.bisulfite && cc == 'C') c = cc = 'T';
 		if(cat == 1) {
@@ -134,18 +156,18 @@ RefRecord fastaRefReadSize(FileBuf& in,
 			if(bpout != NULL) {
 				if(rparms.color) {
 					// output color
-					bpout->write(dinuc2color[charToDna5[(int)c]][lc]);
+					bpout->write(dinuc2color[asc2dna[(int)c]][lc]);
 				} else if(!rparms.color) {
 					// output nucleotide
-					bpout->write(charToDna5[c]);
+					bpout->write(asc2dna[c]);
 				}
 			}
-			lc = charToDna5[(int)c];
-		} else if(cat == 2) {
+			lc = asc2dna[(int)c];
+		} else if(cat >= 2) {
 			// It's an N or a gap
 			lastc = c;
 			assert(cc != 'A' && cc != 'C' && cc != 'G' && cc != 'T');
-			return RefRecord(off, len, first);
+			return RefRecord((uint32_t)off, (uint32_t)len, first);
 		} else {
 			// Not DNA and not a gap, ignore it
 #ifndef NDEBUG
@@ -162,31 +184,37 @@ RefRecord fastaRefReadSize(FileBuf& in,
 		c = in.get();
 	}
 	lastc = c;
-	return RefRecord(off, len, first);
+	return RefRecord((uint32_t)off, (uint32_t)len, first);
 }
 
+#if 0
 static void
-printRecords(ostream& os, const vector<RefRecord>& l) {
+printRecords(ostream& os, const EList<RefRecord>& l) {
 	for(size_t i = 0; i < l.size(); i++) {
 		os << l[i].first << ", " << l[i].off << ", " << l[i].len << endl;
 	}
 }
+#endif
 
 /**
  * Reverse the 'src' list of RefRecords into the 'dst' list.  Don't
  * modify 'src'.
  */
-void reverseRefRecords(const vector<RefRecord>& src,
-					   vector<RefRecord>& dst,
-					   bool recursive,
-					   bool verbose)
+void reverseRefRecords(
+	const EList<RefRecord>& src,
+	EList<RefRecord>& dst,
+	bool recursive,
+	bool verbose)
 {
 	dst.clear();
 	{
-		vector<RefRecord> cur;
-		for(int i = src.size()-1; i >= 0; i--) {
+		EList<RefRecord> cur;
+		for(int i = (int)src.size()-1; i >= 0; i--) {
 			bool first = (i == (int)src.size()-1 || src[i+1].first);
-			if(src[i].len) {
+			// Clause after the || on next line is to deal with empty FASTA
+			// records at the end of the 'src' list, which would be wrongly
+			// omitted otherwise.
+			if(src[i].len || (first && src[i].off == 0)) {
 				cur.push_back(RefRecord(0, src[i].len, first));
 				first = false;
 			}
@@ -202,15 +230,27 @@ void reverseRefRecords(const vector<RefRecord>& src,
 			}
 		}
 	}
-	if(verbose) {
-		cout << "Source: " << endl;
-		printRecords(cout, src);
-		cout << "Dest: " << endl;
-		printRecords(cout, dst);
-	}
+	//if(verbose) {
+	//	cout << "Source: " << endl;
+	//	printRecords(cout, src);
+	//	cout << "Dest: " << endl;
+	//	printRecords(cout, dst);
+	//}
 #ifndef NDEBUG
+	size_t srcnfirst = 0, dstnfirst = 0;
+	for(size_t i = 0; i < src.size(); i++) {
+		if(src[i].first) {
+			srcnfirst++;
+		}
+	}
+	for(size_t i = 0; i < dst.size(); i++) {
+		if(dst[i].first) {
+			dstnfirst++;
+		}
+	}
+	assert_eq(srcnfirst, dstnfirst);
 	if(!recursive) {
-		vector<RefRecord> tmp;
+		EList<RefRecord> tmp;
 		reverseRefRecords(dst, tmp, true);
 		assert_eq(tmp.size(), src.size());
 		for(size_t i = 0; i < src.size(); i++) {
@@ -228,17 +268,16 @@ void reverseRefRecords(const vector<RefRecord>& src,
  * all references combined.  Rewinds each istream before returning.
  */
 std::pair<size_t, size_t>
-fastaRefReadSizes(vector<FileBuf*>& in,
-                  vector<RefRecord>& recs,
-                  vector<uint32_t>& plens,
-                  const RefReadInParams& rparms,
-                  BitpairOutFileBuf* bpout,
-                  int& numSeqs)
+fastaRefReadSizes(
+	EList<FileBuf*>& in,
+	EList<RefRecord>& recs,
+	const RefReadInParams& rparms,
+	BitpairOutFileBuf* bpout,
+	int& numSeqs)
 {
 	uint32_t unambigTot = 0;
 	uint32_t bothTot = 0;
 	assert_gt(in.size(), 0);
-	uint32_t both = 0, unambig = 0;
 	// For each input istream
 	for(size_t i = 0; i < in.size(); i++) {
 		bool first = true;
@@ -246,17 +285,6 @@ fastaRefReadSizes(vector<FileBuf*>& in,
 		// For each pattern in this istream
 		while(!in[i]->eof()) {
 			RefRecord rec = fastaRefReadSize(*in[i], rparms, first, bpout);
-			// Update plens
-			if(rec.first) {
-				if(unambig > 0) {
-					plens.push_back(both);
-				}
-				both = 0;
-				unambig = 0;
-			}
-#ifndef ACCOUNT_FOR_ALL_GAP_REFS
-			if(rec.len == 0) rec.first = false;
-#endif
 			if((unambigTot + rec.len) < unambigTot) {
 				cerr << "Error: Reference sequence has more than 2^32-1 characters!  Please divide the" << endl
 				     << "reference into batches or chunks of about 3.6 billion characters or less each" << endl
@@ -265,9 +293,9 @@ fastaRefReadSizes(vector<FileBuf*>& in,
 			}
 			// Add the length of this record.
 			if(rec.first) numSeqs++;
-			unambigTot += rec.len; unambig += rec.len;
-			bothTot += rec.len;    both += rec.len;
-			bothTot += rec.off;    both += rec.off;
+			unambigTot += rec.len;
+			bothTot += rec.len;
+			bothTot += rec.off;
 			first = false;
 			if(rec.len == 0 && rec.off == 0 && !rec.first) continue;
 			recs.push_back(rec);
@@ -285,9 +313,6 @@ fastaRefReadSizes(vector<FileBuf*>& in,
 	}
 	assert_geq(bothTot, 0);
 	assert_geq(unambigTot, 0);
-	if(unambig > 0) {
-		plens.push_back(both);
-	}
 	return make_pair(
 		unambigTot, // total number of unambiguous DNA characters read
 		bothTot); // total number of DNA characters read, incl. ambiguous ones
