@@ -1,8 +1,20 @@
 /*
- * shmem.h
+ * Copyright 2011, Ben Langmead <langmea@cs.jhu.edu>
  *
- *  Created on: Aug 13, 2009
- *      Author: Ben Langmead
+ * This file is part of Bowtie 2.
+ *
+ * Bowtie 2 is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Bowtie 2 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Bowtie 2.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef SHMEM_H_
@@ -10,7 +22,6 @@
 
 #ifdef BOWTIE_SHARED_MEM
 
-#include <iostream>
 #include <string>
 #include <sys/shm.h>
 #include <unistd.h>
@@ -19,13 +30,11 @@
 #include <stdint.h>
 #include <stdexcept>
 #include "str_util.h"
-#include "btypes.h"
 
 extern void notifySharedMem(void *mem, size_t len);
 
 extern void waitSharedMem(void *mem, size_t len);
 
-#define ALLOC_SHARED_U allocSharedMem<TIndexOffU>
 #define ALLOC_SHARED_U8 allocSharedMem<uint8_t>
 #define ALLOC_SHARED_U32 allocSharedMem<uint32_t>
 #define FREE_SHARED shmdt
@@ -68,7 +77,7 @@ bool allocSharedMem(std::string fname,
 				cerr << "EEXIST" << endl;
 			} else if(errno == EINVAL) {
 				cerr << "Warning: shared-memory chunk's segment size doesn't match expected size (" << (shmemLen) << ")" << endl
-					 << "Deleting old shared memory block and trying again." << endl;
+					 << "Deleteing old shared memory block and trying again." << endl;
 				shmid = shmget(key, 0, 0);
 				if((ret = shmctl(shmid, IPC_RMID, &ds)) < 0) {
 					cerr << "shmctl returned " << ret
@@ -103,10 +112,10 @@ bool allocSharedMem(std::string fname,
 			cerr << "shmctl returned " << ret << " for IPC_STAT and errno is " << errno << endl;
 			throw 1;
 		}
-		if((size_t)ds.shm_segsz != shmemLen) {
+		if(ds.shm_segsz != shmemLen) {
 			cerr << "Warning: shared-memory chunk's segment size (" << ds.shm_segsz
 				 << ") doesn't match expected size (" << shmemLen << ")" << endl
-				 << "Deleting old shared memory block and trying again." << endl;
+				 << "Deleteing old shared memory block and trying again." << endl;
 			if((ret = shmctl(shmid, IPC_RMID, &ds)) < 0) {
 				cerr << "shmctl returned " << ret << " for IPC_RMID and errno is " << errno << endl;
 				throw 1;
@@ -138,7 +147,6 @@ bool allocSharedMem(std::string fname,
 
 #else
 
-#define ALLOC_SHARED_U(...) 0
 #define ALLOC_SHARED_U8(...) 0
 #define ALLOC_SHARED_U32(...) 0
 #define FREE_SHARED(...)
